@@ -22,8 +22,13 @@
 
 #include <QScopedPointer>
 
+class QString;
+
+namespace OpenOrienteering {
+
 class AutosavePrivate;
 
+
 /**
  * @brief Class Autosave implements autosaving behaviour.
  * 
@@ -81,4 +86,7 @@ private:
 	QScopedPointer<AutosavePrivate> autosave_controller;
 };
 
+
+}  // namespace OpenOrienteering
+
 #endif
