@@ -22,50 +22,67 @@
 #ifndef OPENORIENTEERING_MAP_H
 #define OPENORIENTEERING_MAP_H
 
-#include <vector>
+#include <algorithm>
+#include <cstddef>
+#include <functional>
 #include <set>
+#include <vector>
 
+#include <QtGlobal>
+#include <QExplicitlySharedDataPointer>
+#include <QFlags>
 #include <QHash>
+#include <QMetaType>
 #include <QObject>
-#include <QRect>
+#include <QPointer>
+#include <QRectF>
 #include <QScopedPointer>
 #include <QSharedData>
 #include <QString>
+#include <QTransform>
 
-#include "global.h"
-#include "map_coord.h"
-#include "map_grid.h"
-#include "fileformats/file_format.h"
-#include "map_part.h"
+#include "core/map_coord.h"
+#include "core/map_grid.h"
+#include "core/map_part.h"
 
-QT_BEGIN_NAMESPACE
 class QIODevice;
 class QPainter;
+class QTranslator;
 class QWidget;
-class QXmlStreamReader;
-class QXmlStreamWriter;
-QT_END_NAMESPACE
+// IWYU pragma: no_forward_declare QRectF
 
-class Map;
+namespace OpenOrienteering {
+
+class CombinedSymbol;
+class FileFormat;
+class Georeferencing;
+class LineSymbol;
 class MapColor;
 class MapColorMap;
-class MapWidget;
-class MapView;
 class MapPrinterConfig;
-class Symbol;
-class CombinedSymbol;
-class LineSymbol;
-class PointSymbol;
-class Object;
-class RenderConfig;
 class MapRenderables;
+class MapView;
+class MapWidget;
+class Object;
+class PointSymbol;
+class RenderConfig;
+class Symbol;
 class Template;
 class TextSymbol;
 class UndoManager;
 class UndoStep;
-class OCAD8FileImport;
-class Georeferencing;
-class MapGrid;
+
+
+/**
+ * The translator for color and symbol texts.
+ * 
+ * This translator is used by class Map but kept outside of the
+ * class' namespace in order to allow for forward declaration
+ * instead of including "map.h".
+ */
+extern QPointer<QTranslator> map_symbol_translator;
+
+
 
 /** Central class for an OpenOrienteering map */
 class Map : public QObject
@@ -74,7 +91,6 @@ Q_OBJECT
 friend class MapTest;
 friend class MapRenderables;
 friend class OCAD8FileImport;
-friend class XMLFileImport;
 friend class NativeFileImport;
 friend class NativeFileExport;
 friend class XMLFileImporter;
@@ -115,11 +131,13 @@ public:
 	Map();
 	
 	/** Destroys the map. */
-	~Map();
+	~Map() override;
 	
 	
 	/**
 	 * Deletes all map data.
+	 * 
+	 * The resulting map must not be modified before another init().
 	 */
 	void clear();
 	
@@ -132,6 +150,8 @@ public:
 	
 	/**
 	 * Deletes all map data, and reinitializes the empty map.
+	 * 
+	 * This method combines a call to clear() followed by init().
 	 */
 	void reset();
 	
@@ -195,6 +215,26 @@ public:
 	        QHash<const Symbol*, Symbol*>* out_symbol_map = nullptr
 	);
 	
+	/**
+	 * Imports another map into this map with the following strategy:
+	 *  - If the other map contains objects, import all objects with the
+	 *    minimum amount of colors and symbols needed to display them.
+	 *  - If the other map does not contain objects, import all symbols
+	 *    with the minimum amount of colors needed to display them.
+	 *  - If the other map does neither contain objects nor symbols,
+	 *    import all colors.
+	 * The transform is applied to all imported objects.
+	 */
+	QHash<const Symbol*, Symbol*> importMap(
+	        const Map& imported_map,
+	        ImportMode mode,
+	        std::vector<bool>* filter = nullptr,
+	        int symbol_insert_pos = -1,
+	        bool merge_duplicate_symbols = true,
+	        const QTransform& transform = {}
+	);
+	
+	
 	/**
 	 * Serializes the map directly into the given IO device in a known format.
 	 * This can be imported again using importFromIODevice().
@@ -251,7 +291,7 @@ public:
 	 * @param on_screen If true, uses a cosmetic pen (one pixel wide),
 	 *                  otherwise uses a 0.1 mm wide pen.
 	 */
-	void drawGrid(QPainter* painter, QRectF bounding_box, bool on_screen);
+	void drawGrid(QPainter* painter, const QRectF& bounding_box, bool on_screen);
 	
 	/**
 	 * Draws the templates with indices first_template until last_template which
@@ -270,7 +310,7 @@ public:
 	 * @param on_screen Potentially enables some drawing optimizations which
 	 *     decrease drawing quality. Should be enabled when drawing on-screen.
 	 */
-	void drawTemplates(QPainter* painter, QRectF bounding_box, int first_template,
+	void drawTemplates(QPainter* painter, const QRectF& bounding_box, int first_template,
 					   int last_template, const MapView* view, bool on_screen) const;
 	
 	
@@ -331,7 +371,7 @@ public:
 	 *     3 for pixel_border.
 	 * @param do_update Whether a repaint of the covered area should be triggered.
 	 */
-	void setDrawingBoundingBox(QRectF map_coords_rect, int pixel_border, bool do_update = true);
+	void setDrawingBoundingBox(const QRectF& map_coords_rect, int pixel_border, bool do_update = true);
 	
 	/**
 	 * Removes the drawing bounding box and triggers a repaint. Use this if
@@ -344,7 +384,7 @@ public:
 	 * This is the analogon to setDrawingBoundingBox() for activities.
 	 * See setDrawingBoundingBox().
 	 */
-	void setActivityBoundingBox(QRectF map_coords_rect, int pixel_border, bool do_update = true);
+	void setActivityBoundingBox(const QRectF& map_coords_rect, int pixel_border, bool do_update = true);
 	
 	/**
 	 * This is the analogon to clearDrawingBoundingBox() for activities.
@@ -359,7 +399,22 @@ public:
 	 * 
 	 * See setDrawingBoundingBox() and setActivityBoundingBox().
 	 */
-	void updateDrawing(QRectF map_coords_rect, int pixel_border);
+	void updateDrawing(const QRectF& map_coords_rect, int pixel_border);
+	
+	
+	
+	// Element translations
+	
+	/**
+	 * Returns a translated symbol text (name or description), or the original text.
+	 */
+	QString translate(const QString& symbol_text) const;
+	
+	/**
+	 * Returns a translated symbol text (name or description), or an empty string.
+	 */
+	QString raw_translation(const QString& symbol_text) const;
+	
 	
 	
 	// Colors
@@ -418,7 +473,7 @@ public:
 	
 	/**
 	 * Marks the colors as "dirty", i.e. as having unsaved changes.
-	 * Emits hasUnsavedChanges(true) if the map did not have unsaved changed before.
+	 * Emits hasUnsavedChanged(true) if the map did not have unsaved changed before.
 	 */
 	void setColorsDirty();
 	
@@ -451,6 +506,13 @@ public:
 	
 	// Symbols
 	
+	/** Returns the symbol set ID. */
+	QString symbolSetId() const;
+	
+	/** Sets the symbol set ID. */
+	void setSymbolSetId(const QString& id);
+	
+	
 	/** Returns the number of symbols in this map. */
 	int getNumSymbols() const;
 	
@@ -497,7 +559,7 @@ public:
 	
 	/**
 	 * Marks the symbols as "dirty", i.e. as having unsaved changes.
-	 * Emits hasUnsavedChanges(true) if the map did not have unsaved changed before.
+	 * Emits hasUnsavedChanged(true) if the map did not have unsaved changed before.
 	 */
 	void setSymbolsDirty();
 	
@@ -526,7 +588,25 @@ public:
 	 */
 	void determineSymbolUseClosure(std::vector< bool >& symbol_bitfield) const;
 	
+	/**
+	 * Returns the scale factor to be used for default symbol icons.
+	 * 
+	 * The full icon size (width, height) is represented by 1.0.
+	 */
+	qreal symbolIconZoom() const;
 	
+public slots:
+	/**
+	 * Updates the symbol icon zoom from the current set of symbols.
+	 * 
+	 * The symbol icon zoom is chosen so that most symbols fit into the full
+	 * icon space, and the number of symbol below 10% size is kept low.
+	 * For a map without symbols, this returns 1.0.
+	 */
+	void updateSymbolIconZoom();
+	
+	
+public:
 	// Templates
 	
 	/** Returns the number of templates in this map. */
@@ -581,7 +661,7 @@ public:
 	 * Warning: does nothing if the template is not visible in a widget!
 	 * So make sure to call this and showing/hiding a template in the correct order!
 	 */
-	void setTemplateAreaDirty(Template* temp, QRectF area, int pixel_border);
+	void setTemplateAreaDirty(Template* temp, const QRectF& area, int pixel_border);
 	
 	/**
 	 * Marks the whole area of the i-th template as "to be repainted".
@@ -599,7 +679,7 @@ public:
 	
 	/**
 	 * Marks the template settings as "dirty", i.e. as having unsaved changes.
-	 * Emits hasUnsavedChanges(true) if the map did not have unsaved changed before.
+	 * Emits hasUnsavedChanged(true) if the map did not have unsaved changed before.
 	 */
 	void setTemplatesDirty();
 	
@@ -777,7 +857,7 @@ public:
 	
 	/**
 	 * Marks the objects as "dirty", i.e. as having unsaved changes.
-	 * Emits hasUnsavedChanges(true) if the map did not have unsaved changed before.
+	 * Emits hasUnsavedChanged(true) if the map did not have unsaved changed before.
 	 */
 	void setObjectsDirty();
 	
@@ -785,7 +865,7 @@ public:
 	 * Marks the area given by map_coords_rect as "dirty" in all map widgets,
 	 * i.e. as needing to be redrawn because some object(s) changed there.
 	 */
-	void setObjectAreaDirty(QRectF map_coords_rect);
+	void setObjectAreaDirty(const QRectF& map_coords_rect);
 	
 	/**
 	 * Finds and returns all objects at the given position in the current part.
@@ -846,31 +926,36 @@ public:
 	 * @param map_coord_rect The query rect.
 	 * @param include_hidden_objects Set to true if you want to find hidden objects.
 	 */
-	int countObjectsInRect(QRectF map_coord_rect, bool include_hidden_objects);
+	int countObjectsInRect(const QRectF& map_coord_rect, bool include_hidden_objects);
+	
 	
 	/**
-	 * Applies a condition on all objects (until the first match is found).
+	 * Applies a condition on all objects until the first match is found.
 	 * 
 	 * @return True if there is an object matching the condition, false otherwise.
 	 */
-	template<typename Condition>
-	bool existsObject(const Condition& condition) const;
+	bool existsObject(const std::function<bool (const Object*)>& condition) const;
 	
 	/**
 	 * Applies an operation on all objects which match a particular condition.
-	 * 
-	 * @return False if the operation fails for any matching object, true otherwise.
 	 */
-	template<typename Operation, typename Condition>
-	bool applyOnMatchingObjects(const Operation& operation, const Condition& condition);
+	void applyOnMatchingObjects(const std::function<void (Object*)>& operation, const std::function<bool (const Object*)>& condition);
+	
+	/**
+	 * Applies an operation on all objects which match a particular condition.
+	 */
+	void applyOnMatchingObjects(const std::function<void (Object*, MapPart*, int)>& operation, const std::function<bool (const Object*)>& condition);
 	
 	/**
 	 * Applies an operation on all objects.
-	 * 
-	 * @return False if the operation fails for any object, true otherwise.
 	 */
-	template<typename Operation>
-	bool applyOnAllObjects(const Operation& operation);
+	void applyOnAllObjects(const std::function<void (Object*)>& operation);
+	
+	/**
+	 * Applies an operation on all objects.
+	 */
+	void applyOnAllObjects(const std::function<void (Object*, MapPart*, int)>& operation);
+	
 	
 	/** Scales all objects by the given factor. */
 	void scaleAllObjects(double factor, const MapCoord& scaling_center);
@@ -1194,7 +1279,7 @@ public:
 	 * setHasUnsavedChanges() alone followed by a map change and an undo would
 	 * result in no changed flag.
 	 */
-	bool hasUnsavedChanged() const;
+	bool hasUnsavedChanges() const;
 	
 	/** Do not use this in usual cases, see hasUnsavedChanged(). */
 	void setHasUnsavedChanges(bool has_unsaved_changes);
@@ -1216,7 +1301,7 @@ public:
 	
 	/**
 	 * Marks somthing unspecific in the map as "dirty", i.e. as having unsaved changes.
-	 * Emits hasUnsavedChanges(true) if the map did not have unsaved changed before.
+	 * Emits hasUnsavedChanged(true) if the map did not have unsaved changed before.
 	 * 
 	 * Use setColorsDirty(), setSymbolsDirty(), setTemplatesDirty() or
 	 * setObjectsDirty() if you know more specificly what has changed.
@@ -1261,7 +1346,7 @@ signals:
 	/**
 	 * Emitted when a the map enters or leaves the state which is saved on map.
 	 */
-	void hasUnsavedChanges(bool is_clean);
+	void hasUnsavedChanged(bool is_clean);
 	
 	
 	/** Emitted when a color is added to the map, gives the color's index and pointer. */
@@ -1289,12 +1374,15 @@ signals:
 	/** Emitted when a symbol in the map is deleted. */
 	void symbolDeleted(int pos, const Symbol* old_symbol);
 	
+	/** Emitted when the symbol icon zoom changes. */
+	void symbolIconZoomChanged();
+	
 	
 	/** Emitted when a template is added to the map, gives the template's index and pointer. */
-	void templateAdded(int pos, const Template* temp);
+	void templateAdded(int pos, Template* temp);
 	
 	/** Emitted when a template in the map is changed, gives the template's index and pointer. */
-	void templateChanged(int pos, const Template* temp);
+	void templateChanged(int pos, Template* temp);
 	
 	/** Emitted when a template in the map is deleted, gives the template's index and pointer. */
 	void templateDeleted(int pos, const Template* old_temp);
@@ -1396,18 +1484,23 @@ private:
 		void adjustColorPriorities(int first, int last);
 	};
 	
-	/// Imports the other symbol set into this set, only importing the symbols for which filter[color_index] == true and
-	/// returning the map from symbol indices in other to imported indices. Imported symbols are placed after the existing symbols.
-	void importSymbols(
-	        const Map* other,
+	
+	/** 
+	 * Imports the other symbol set into this map's symbols.
+	 * 
+	 * If a filter is given, imports only the symbols for which filter[color_index] == true.
+	 * Imported symbols are placed at insert_pos (if positive), or after the existing symbols.
+	 * Returns a mapping from original symbols (in other) to imported symbols.
+	 */
+	QHash<const Symbol*, Symbol*> importSymbols(
+	        const Map& other,
 	        const MapColorMap& color_map,
 	        int insert_pos = -1,
 	        bool merge_duplicates = true,
-	        std::vector<bool>* filter = nullptr,
-	        QHash<int, int>* out_indexmap = nullptr,
-	        QHash<const Symbol*, Symbol*>* out_pointermap = nullptr
+	        const std::vector<bool>& filter = {}
 	);
 	
+	
 	void addSelectionRenderables(const Object* object);
 	void updateSelectionRenderables(const Object* object);
 	void removeSelectionRenderables(const Object* object);
@@ -1416,15 +1509,17 @@ private:
 	
 	QExplicitlySharedDataPointer<MapColorSet> color_set;
 	bool has_spot_colors;
+	QString symbol_set_id;
 	SymbolVector symbols;
+	mutable qreal symbol_icon_scale = 0;
 	TemplateVector templates;
 	TemplateVector closed_templates;
-	int first_front_template;		// index of the first template in templates which should be drawn in front of the map
+	int first_front_template = 0;		// index of the first template in templates which should be drawn in front of the map
 	PartVector parts;
 	ObjectSelection object_selection;
-	Object* first_selected_object;
+	Object* first_selected_object = nullptr;
 	QScopedPointer<UndoManager> undo_manager;
-	std::size_t current_part_index;
+	std::size_t current_part_index = 0;
 	WidgetVector widgets;
 	QScopedPointer<MapRenderables> renderables;
 	QScopedPointer<MapRenderables> selection_renderables;
@@ -1468,13 +1563,10 @@ private:
 	static CombinedSymbol* covering_combined_line;
 };
 
-Q_DECLARE_METATYPE(const Map*)
 
 
 // ### Map inline code ###
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(Map::ImportMode)
-
 inline
 int Map::getNumColors() const
 {
@@ -1519,6 +1611,15 @@ const MapColor* Map::getColor(int i) const
 	}
 }
 
+
+
+inline
+QString Map::symbolSetId() const
+{
+	return symbol_set_id;
+}
+
+
 inline
 int Map::getNumSymbols() const
 {
@@ -1529,7 +1630,7 @@ template<typename T>
 void Map::sortSymbols(T compare)
 {
 	std::stable_sort(symbols.begin(), symbols.end(), compare);
-	// TODO: emit(symbolChanged(pos, symbol)); ? s/b same choice as for moveSymbol()
+	// TODO: emit symbolChanged(pos, symbol); ? s/b same choice as for moveSymbol()
 	setSymbolsDirty();
 }
 
@@ -1647,33 +1748,6 @@ Map::ObjectSelection::const_iterator Map::selectedObjectsEnd() const
 	return object_selection.cend();
 }
 
-template<typename Condition>
-bool Map::existsObject(const Condition& condition) const
-{
-	for (const MapPart* part : parts)
-		if (part->existsObject<Condition>(condition))
-			return true;
-	return false;
-}
-
-template<typename Operation, typename Condition>
-bool Map::applyOnMatchingObjects(const Operation& operation, const Condition& condition)
-{
-	bool result = true;
-	for (MapPart* part : parts)
-		result &= part->applyOnMatchingObjects(operation, condition);
-	return result;
-}
-
-template<typename Operation>
-bool Map::applyOnAllObjects(const Operation& operation)
-{
-	bool result = true;
-	for (MapPart* part : parts)
-		result &= part->applyOnAllObjects(operation);
-	return result;
-}
-
 inline
 const Object* Map::getFirstSelectedObject() const
 {
@@ -1717,7 +1791,7 @@ bool Map::hasPrinterConfig()
 }
 
 inline
-bool Map::hasUnsavedChanged() const
+bool Map::hasUnsavedChanges() const
 {
 	return unsaved_changes;
 }
@@ -1812,4 +1886,13 @@ TextSymbol* Map::getUndefinedText()
 	return undefined_text;
 }
 
+
+}  // namespace OpenOrienteering
+
+
+Q_DECLARE_METATYPE(const OpenOrienteering::Map*)
+
+Q_DECLARE_OPERATORS_FOR_FLAGS(OpenOrienteering::Map::ImportMode)
+
+
 #endif
