@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2013-2015 Kai Pastor
+ *    Copyright 2013-2017 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -19,21 +19,24 @@
  */
 
 
-#ifndef _OPENORIENTEERING_MAP_COORD_H_
-#define _OPENORIENTEERING_MAP_COORD_H_
+#ifndef OPENORIENTEERING_MAP_COORD_H
+#define OPENORIENTEERING_MAP_COORD_H
 
 #include <cmath>
 #include <vector>
 
+#include <QtGlobal>
 #include <QCoreApplication>
 #include <QFlags>
+#include <QPoint>
 #include <QPointF>
+#include <QString>
 
-class QString;
-class QTextStream;
 class QXmlStreamReader;
 class QXmlStreamWriter;
 
+namespace OpenOrienteering {
+
 #ifndef NO_NATIVE_FILE_FORMAT
 	
 /**
@@ -81,7 +84,7 @@ struct LegacyMapCoord
  */
 class MapCoord
 {
-	Q_DECLARE_TR_FUNCTIONS(MapCoord)
+	Q_DECLARE_TR_FUNCTIONS(OpenOrienteering::MapCoord)
 	
 public:
 	/**
@@ -174,7 +177,7 @@ public:
 	 * 
 	 * @deprecated
 	 */
-	MapCoord(const LegacyMapCoord& coord);
+	MapCoord(const LegacyMapCoord& coord) noexcept;
 
 #endif	
 	
@@ -186,7 +189,7 @@ public:
 	 * you need to use other argument types than int if the compiler complains
 	 * about ambiguity.
 	 */
-	constexpr MapCoord(int x, int y);
+	constexpr MapCoord(int x, int y) noexcept;
 	
 private:
 	/** Creates a MapCoord from native map coordinates.
@@ -194,27 +197,51 @@ private:
 	 * This is exposed via fromNative() - you must be explicit when you want to
 	 * use native units.
 	 */
-	constexpr MapCoord(qint32 x, qint32 y, int flags);
+	constexpr MapCoord(qint32 x, qint32 y, Flags flags) noexcept;
+	
+	/** Creates a MapCoord from native map coordinates.
+	 *
+	 * This is exposed via fromNative() - you must be explicit when you want to
+	 * use native units.
+	 */
+	constexpr MapCoord(qint32 x, qint32 y, Flag flag) noexcept;
+	
+	// Prevent selection of MapCoord(qreal, qreal, int)
+	MapCoord(qint32, qint32, int) = delete;
 	
 public:
 	/** Creates a MapCoord from a position given in millimeters on the map. */
-	constexpr MapCoord(qreal x, qreal y);
+	constexpr MapCoord(qreal x, qreal y) noexcept;
 	
 	/** Creates a MapCoord with the given flags from a position given in millimeters on the map. */
-	constexpr MapCoord(qreal x, qreal y, int flags);
+	constexpr MapCoord(qreal x, qreal y, Flags flags) noexcept;
+	
+	/** Creates a MapCoord with the given flags from a position given in millimeters on the map. */
+	constexpr MapCoord(qreal x, qreal y, Flag flag) noexcept;
+	
+	MapCoord(qreal x, qreal y, int flags) = delete;
 	
 	/** Creates a MapCoord with the position taken from a QPointF. */
-	explicit constexpr MapCoord(QPointF point);
+	explicit constexpr MapCoord(QPointF point) noexcept;
 	
 	/** Creates a MapCoord with the given flags and with the position taken from a QPointF. */
-	constexpr MapCoord(QPointF point, int flags);
+	constexpr MapCoord(QPointF point, Flags flags) noexcept;
 	
+	/** Creates a MapCoord with the given flags and with the position taken from a QPointF. */
+	constexpr MapCoord(QPointF point, Flag flag) noexcept;
+	
+	MapCoord(QPointF point, int flags) = delete;
 	
 	/** Creates a MapCoord from native map coordinates. */
-	static constexpr MapCoord fromNative(qint32 x, qint32 y);
+	static constexpr MapCoord fromNative(qint32 x, qint32 y) noexcept;
 	
 	/** Creates a MapCoord from native map coordinates. */
-	static constexpr MapCoord fromNative(qint32 x, qint32 y, int flags);
+	static constexpr MapCoord fromNative(qint32 x, qint32 y, Flags flags) noexcept;
+	
+	/** Creates a MapCoord from native map coordinates. */
+	static constexpr MapCoord fromNative(qint32 x, qint32 y, Flag flag) noexcept;
+	
+	static MapCoord fromNative(qint32 x, qint32 y, int flags) = delete;
 	
 	/** It is illegal to call MapCoord::fromNative with qint64 arguments.
 	 *
@@ -270,10 +297,10 @@ public:
 	
 	
 	/** Returns the coord's flags separately, merged into the lowest 8 bits of an int. */
-	constexpr int flags() const;
+	constexpr Flags::Int flags() const noexcept;
 	
 	/** Sets the flags as retrieved by flags(). */
-	void setFlags(int flags);
+	void setFlags(Flags::Int flags) noexcept;
 	
 	
 	/**
@@ -413,7 +440,9 @@ public:
 	 * apply the BoundsOffset() and throw a std::range_error if the adjusted
 	 * coordinates are out of bounds for qint32.
 	 */
-	static MapCoord load(qreal x, qreal y, int flags);
+	static MapCoord load(qreal x, qreal y, MapCoord::Flags flags);
+	
+	static MapCoord load(qreal x, qreal y, int flags) = delete;
 	
 	/** Creates a MapCoord from map coordinates in millimeters, with offset handling.
 	 * 
@@ -421,7 +450,9 @@ public:
 	 * apply the BoundsOffset() and throw a std::range_error if the adjusted
 	 * coordinates are out of bounds for qint32.
 	 */
-	static MapCoord load(QPointF p, int flags);
+	static MapCoord load(QPointF p, MapCoord::Flags flags);
+	
+	static MapCoord load(QPointF p, int flags) = delete;
 	
 	
 	friend constexpr bool operator==(const MapCoord& lhs, const MapCoord& rhs);
@@ -632,9 +663,6 @@ typedef std::vector<MapCoordF> MapCoordVectorF;
 
 // ### MapCoord inline code ###
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(MapCoord::Flags)
-
-
 constexpr bool MapCoord::BoundsOffset::isZero() const
 {
 	return x==0 && y==0;
@@ -657,7 +685,7 @@ constexpr MapCoord::MapCoord() noexcept
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(int x, int y)
+constexpr MapCoord::MapCoord(int x, int y) noexcept
  : xp{ x*1000 }
  , yp{ y*1000 }
  , fp{ 0 }
@@ -665,7 +693,7 @@ constexpr MapCoord::MapCoord(int x, int y)
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(qint32 x, qint32 y, int flags)
+constexpr MapCoord::MapCoord(qint32 x, qint32 y, Flags flags) noexcept
  : xp{ x }
  , yp{ y }
  , fp{ flags }
@@ -673,7 +701,15 @@ constexpr MapCoord::MapCoord(qint32 x, qint32 y, int flags)
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(qreal x, qreal y)
+constexpr MapCoord::MapCoord(qint32 x, qint32 y, Flag flag) noexcept
+ : xp{ x }
+ , yp{ y }
+ , fp{ flag }
+{
+	// nothing else
+}
+
+constexpr MapCoord::MapCoord(qreal x, qreal y) noexcept
  : xp{ qRound(x*1000) }
  , yp{ qRound(y*1000) }
  , fp{ 0 }
@@ -681,7 +717,7 @@ constexpr MapCoord::MapCoord(qreal x, qreal y)
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(qreal x, qreal y, int flags)
+constexpr MapCoord::MapCoord(qreal x, qreal y, Flags flags) noexcept
  : xp{ qRound(x*1000) }
  , yp{ qRound(y*1000) }
  , fp{ flags }
@@ -689,28 +725,47 @@ constexpr MapCoord::MapCoord(qreal x, qreal y, int flags)
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(QPointF point)
+constexpr MapCoord::MapCoord(qreal x, qreal y, Flag flag) noexcept
+ : xp{ qRound(x*1000) }
+ , yp{ qRound(y*1000) }
+ , fp{ flag }
+{
+	// nothing else
+}
+
+constexpr MapCoord::MapCoord(QPointF point) noexcept
  : MapCoord { point.x(), point.y() }
 {
 	// nothing else
 }
 
-constexpr MapCoord::MapCoord(QPointF point, int flags)
+constexpr MapCoord::MapCoord(QPointF point, Flags flags) noexcept
  : MapCoord { point.x(), point.y(), flags }
 {
 	// nothing else
 }
 
-constexpr MapCoord MapCoord::fromNative(qint32 x, qint32 y)
+constexpr MapCoord::MapCoord(QPointF point, Flag flag) noexcept
+ : MapCoord { point.x(), point.y(), flag }
+{
+	// nothing else
+}
+
+constexpr MapCoord MapCoord::fromNative(qint32 x, qint32 y) noexcept
 {
 	return MapCoord{ x, y, Flags() };
 }
 
-constexpr MapCoord MapCoord::fromNative(qint32 x, qint32 y, int flags)
+constexpr MapCoord MapCoord::fromNative(qint32 x, qint32 y, Flags flags) noexcept
 {
 	return MapCoord{ x, y, flags };
 }
 
+constexpr MapCoord MapCoord::fromNative(qint32 x, qint32 y, Flag flag) noexcept
+{
+	return MapCoord{ x, y, flag };
+}
+
 constexpr qreal MapCoord::x() const
 {
 	return nativeX() / 1000.0;
@@ -755,13 +810,13 @@ void MapCoord::setNativeY(qint32 new_y)
 	yp = new_y;
 }
 
-constexpr int MapCoord::flags() const
+constexpr MapCoord::Flags::Int MapCoord::flags() const noexcept
 {
 	return fp;
 }
 
 inline
-void MapCoord::setFlags(int flags)
+void MapCoord::setFlags(Flags::Int flags) noexcept
 {
 	fp = Flags(flags);
 }
@@ -1128,5 +1183,10 @@ constexpr const MapCoordF operator/(const MapCoordF& lhs, qreal divisor)
 }
 
 
+}  // namespace OpenOrienteering
+
+
+Q_DECLARE_OPERATORS_FOR_FLAGS(OpenOrienteering::MapCoord::Flags)
+
 
 #endif
