@@ -21,9 +21,18 @@
 
 #include "path_coord.h"
 
-#include "virtual_path.h"
-#include "util/util.h"
+#include <algorithm>
+#include <cstddef>
+#include <iterator>
+#include <memory>
+#include <type_traits>
 
+#include "core/map_coord.h"
+#include "core/virtual_coord_vector.h"
+#include "core/virtual_path.h"
+
+
+namespace OpenOrienteering {
 
 static_assert(std::is_nothrow_default_constructible<PathCoord>::value,
               "PathCoord must be nothrow default constructible.");
@@ -100,7 +109,7 @@ MapCoordF SplitPathCoord::tangentVector() const
 	}
 	
 	// Search along current edge
-	if (index < last_index && param != 0.0)
+	if (index < last_index && param != 0.0f)
 	{
 		do
 		{
@@ -109,7 +118,7 @@ MapCoordF SplitPathCoord::tangentVector() const
 			if (pos.distanceSquaredTo(next) >= PathCoord::tangentEpsilonSquared())
 				goto next_found;
 		}
-		while (path_coords[index].param != 0.0);
+		while (path_coords[index].param != 0.0f);
 	}
 		
 	// Attention, switching from PathCoordVector index to MapCoordVectorF index.
@@ -183,9 +192,9 @@ next_found:
 	index = path_coord_index;
 	
 	// Search along curve
-	if (param != 0.0)
+	if (param != 0.0f)
 	{
-		while (path_coords[index].param != 0.0)
+		while (path_coords[index].param != 0.0f)
 		{
 			--index;
 			prev = path_coords[index].pos;
@@ -222,9 +231,10 @@ next_found:
 		// Search along curve
 		Q_ASSERT(index > path_coords[path_coord_index].index);
 		// Attention, switching from MapCoordVectorF index to PathCoordVector index.
-		index = std::upper_bound(std::begin(path_coords)+path_coord_index, std::end(path_coords)-1, index, PathCoord::valueLessThanIndex)->index;
+		auto pc = std::upper_bound(std::begin(path_coords)+path_coord_index, std::end(path_coords)-1, index, PathCoord::valueLessThanIndex);
+		index = std::distance(std::begin(path_coords), pc);
 		last_index = path_coord_index + 1;
-		while (index > path_coord_index)
+		while (index > last_index)
 		{
 			--index;
 			prev = path_coords[index].pos;
@@ -407,18 +417,18 @@ SplitPathCoord SplitPathCoord::at(
 			factor = qBound(0.0f, (length - prev_coord.clen) / curve_length, 1.0f);
 			auto prev_param    = prev_coord.param;
 			auto current_param = current_coord.param;
-			if (current_param == 0.0)
-				current_param = 1.0;
+			if (current_param == 0.0f)
+				current_param = 1.0f;
 			split.param = prev_param + (current_param - prev_param) * factor;
-			Q_ASSERT(split.param >= 0.0 && split.param <= 1.0);
-			if (split.param == 1.0)
-				split.param = 0.0;
+			Q_ASSERT(split.param >= 0.0f && split.param <= 1.0f);
+			if (split.param == 1.0f)
+				split.param = 0.0f;
 		}
 		
 		if (!split.is_curve_end)
 		{
 			// Straight
-			split.pos = prev_coord.pos + factor * (current_coord.pos - prev_coord.pos);
+			split.pos = prev_coord.pos + qreal(factor) * (current_coord.pos - prev_coord.pos);
 			
 			if (current_coord.index > path_coords.front().index)
 				split.curve_end[1] = coords[current_coord.index-1];
@@ -427,7 +437,7 @@ SplitPathCoord SplitPathCoord::at(
 			
 			split.curve_start[0] = current_coord.pos;
 		}
-		else if (split.param == 0.0)
+		else if (split.param == 0.0f)
 		{
 			// At a node, after a curve
 			split.pos          = current_coord.pos;
@@ -473,7 +483,7 @@ SplitPathCoord SplitPathCoord::at(
 			}
 		}
 		
-		if (split.param == 0.0)
+		if (split.param == 0.0f)
 		{
 			// Handle curve_start for non-in-bezier splits.
 			split.is_curve_start = flags[current_coord.index].isCurveStart();
@@ -514,3 +524,6 @@ bool PathCoord::valueLessThanIndex(size_type value, const PathCoord& coord)
 {
 	return value < coord.index;
 }
+
+
+}  // namespace OpenOrienteering
