@@ -20,21 +20,27 @@
 
 #include "gdal_settings_page.h"
 
+#include <QtGlobal>
+#include <QAbstractItemModel>
 #include <QCheckBox>
 #include <QFormLayout>
 #include <QHeaderView>
+#include <QLabel>
+#include <QSignalBlocker>
+#include <QSpacerItem>
+#include <QStringList>
 #include <QTableWidget>
+#include <QTableWidgetItem>
 #include <QVBoxLayout>
 
 #include "fileformats/file_format_registry.h"
+#include "gdal/gdal_manager.h"
+#include "gdal/ogr_file_format.h"
 #include "gui/util_gui.h"
 #include "util/backports.h"
-#include "util/scoped_signals_blocker.h"
-
-#include "gdal_manager.h"
-#include "ogr_file_format.h"
 
 
+namespace OpenOrienteering {
 
 GdalSettingsPage::GdalSettingsPage(QWidget* parent)
 : SettingsPage(parent)
@@ -222,3 +228,6 @@ int GdalSettingsPage::findDuplicateKey(const QString& key, int row) const
 	}
 	return row;
 }
+
+
+}  // namespace OpenOrienteering
