@@ -20,8 +20,19 @@
 #ifndef OPENORIENTEERING_OGR_FILE_FORMAT_H
 #define OPENORIENTEERING_OGR_FILE_FORMAT_H
 
+#include <cstddef>
+
 #include "fileformats/file_format.h"
 
+class QIODevice;
+
+namespace OpenOrienteering {
+
+class Importer;
+class Map;
+class MapView;
+
+
 /**
  * A FileFormat for geospatial vector data supported by OGR.
  * 
@@ -43,7 +54,7 @@ public:
 	 * 
 	 * There is no cheap way to determine the answer via OGR.
 	 */
-	bool understands(const unsigned char *, size_t) const override;
+	bool understands(const unsigned char* buffer, std::size_t size) const override;
 	
 	/**
 	 * Creates an importer object and configures it for the given input stream
@@ -52,4 +63,7 @@ public:
 	Importer* createImporter(QIODevice* stream, Map *map, MapView *view) const override;
 };
 
+
+}  // namespace OpenOrienteering
+
 #endif // OPENORIENTEERING_OGR_FILE_FORMAT_H
