@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013, 2014 Thomas Schöps
- *    Copyright 2013, 2014 Kai Pastor
+ *    Copyright 2013-2017 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -19,45 +19,55 @@
  */
 
 
-#ifndef _OPENORIENTEERING_MAP_EDITOR_H_
-#define _OPENORIENTEERING_MAP_EDITOR_H_
+#ifndef OPENORIENTEERING_MAP_EDITOR_H
+#define OPENORIENTEERING_MAP_EDITOR_H
 
-#include <vector>
+#include <memory>
 
 #include <QClipboard>
+#include <QHash>
+#include <QObject>
 #include <QPointer>
 #include <QScopedPointer>
+#include <QString>
 #include <QTimer>
 
 #include "gui/main_window_controller.h"
-#include "core/map.h"
 
-QT_BEGIN_NAMESPACE
+class QAction;
+class QByteArray;
 class QComboBox;
 class QDockWidget;
 class QFrame;
+class QKeyEvent;
 class QLabel;
 class QMenu;
 class QSignalMapper;
-class QSizeGrip;
+// IWYU pragma: no_forward_declare QString
 class QToolBar;
 class QToolButton;
-QT_END_NAMESPACE
+class QWidget;
+
+namespace OpenOrienteering {
 
 class ActionGridBar;
 class CompassDisplay;
 class EditorDockWidget;
-class GeoreferencingDialog;
-class Map;
-class MapView;
-class MapWidget;
-class MapEditorActivity;
-class MapEditorTool;
+class FileFormat;
 class GPSDisplay;
 class GPSTemporaryMarkers;
 class GPSTrackRecorder;
+class GeoreferencingDialog;
+class MainWindow;
+class Map;
+class MapEditorActivity;
+class MapEditorTool;
+class MapFindFeature;
+class MapView;
+class MapWidget;
 class PrintWidget;
 class ReopenTemplateDialog;
+class Symbol;
 class SymbolWidget;
 class Template;
 class TemplateListWidget;
@@ -98,7 +108,7 @@ public:
 	MapEditorController(OperatingMode mode, Map* map = nullptr, MapView* map_view = nullptr);
 	
 	/** Destroys the MapEditorController. */
-	~MapEditorController();
+	~MapEditorController() override;
 	
 	/** Returns if the editor is in mobile mode. */
 	bool isInMobileMode() const;
@@ -106,7 +116,7 @@ public:
 	/**
 	 * Changes to new_tool as the new active tool.
 	 * If there is a current tool before, calls deleteLater() on it.
-	 * new_tool may be NULL, but it is unusual to have no active tool, so
+	 * new_tool may be nullptr, but it is unusual to have no active tool, so
 	 * consider setEditTool() instead.
 	 */
 	void setTool(MapEditorTool* new_tool);
@@ -120,7 +130,7 @@ public:
 	/**
 	 * Sets new_override_tool as the new active override tool.
 	 * This takes precedence over all tools set via setTool().
-	 * new_override_tool may be NULL, which disables using an override tool
+	 * new_override_tool may be nullptr, which disables using an override tool
 	 * and re-enables the normal tool set via setTool().
 	 */
 	void setOverrideTool(MapEditorTool* new_override_tool);
@@ -133,12 +143,12 @@ public:
 	
 	
 	/**
-	 * @brief Returns the active symbol, or NULL.
+	 * @brief Returns the active symbol, or nullptr.
 	 * 
 	 * The active symbol is the single symbol which is to be used by drawing
 	 * tools and actions.
 	 * 
-	 * It there is no active symbol, this function returns NULL.
+	 * It there is no active symbol, this function returns nullptr.
 	 */
 	Symbol* activeSymbol() const;
 	
@@ -155,7 +165,7 @@ public:
 	 * Returns true when editing is in progress.
 	 * @see setEditingInProgress
 	 */
-	virtual bool isEditingInProgress() const;
+	bool isEditingInProgress() const override;
 	
 	/**
 	 * Adds a a floating dock widget to the main window.
@@ -165,7 +175,7 @@ public:
 	
 	/**
 	 * Sets the current editor activity.
-	 * new_activity may be NULL to disable the current editor activity.
+	 * new_activity may be nullptr to disable the current editor activity.
 	 */
 	void setEditorActivity(MapEditorActivity* new_activity);
 	
@@ -213,34 +223,34 @@ public:
 	
 	
 	/**
-	 * Returns the action identified by id if it exists, or NULL.
+	 * Returns the action identified by id if it exists, or nullptr.
 	 * This allows the reuse of the controller's actions in dock widgets etc.
 	 */
 	QAction* getAction(const char* id);
 	
 	/** Override from MainWindowController */
-	virtual bool save(const QString& path);
+	bool save(const QString& path) override;
 	/** Override from MainWindowController */
-	virtual bool exportTo(const QString& path, const FileFormat* format = NULL);
+	bool exportTo(const QString& path, const FileFormat* format = nullptr) override;
 	/** Override from MainWindowController */
-	virtual bool load(const QString& path, QWidget* dialog_parent = NULL);
+	bool load(const QString& path, QWidget* dialog_parent = nullptr) override;
 	
 	/** Override from MainWindowController */
-	virtual void attach(MainWindow* window);
+	void attach(MainWindow* window) override;
 	/** Override from MainWindowController */
-	virtual void detach();
+	void detach() override;
 	
 	/**
 	 * @copybrief MainWindowController::keyPressEventFilter
 	 * This implementation passes the event to MapWidget::keyPressEventFilter.
 	 */
-	virtual bool keyPressEventFilter(QKeyEvent* event);
+	bool keyPressEventFilter(QKeyEvent* event) override;
 	
 	/** 
 	 * @copybrief MainWindowController::keyReleaseEventFilter
 	 * This implementation passes the event to MapWidget::keyReleaseEventFilter.
 	 */
-	virtual bool keyReleaseEventFilter(QKeyEvent* event);
+	bool keyReleaseEventFilter(QKeyEvent* event) override;
 	
 public slots:
 	/**
@@ -269,6 +279,8 @@ public slots:
 	
 	/** Activates the pan tool. */
 	void pan();
+	/** Moves view to GPS position. */
+	void moveToGpsPos();
 	/** Zooms in in the current map widget. */
 	void zoomIn();
 	/** Zooms out in the current map widget. */
@@ -294,8 +306,12 @@ public slots:
 	void showSymbolWindow(bool show);
 	/** Shows or hides the color dock widget. */
 	void showColorWindow(bool show);
+	/** Shows a dialog for changing the symbol set ID. */
+	void symbolSetIdClicked();
 	/** Shows the "load symbols from" dialog. */
 	void loadSymbolsFromClicked();
+	/** Loads a CRT file and shows the symbol replacement dialog. */
+	void loadCrtClicked();
 	/** TODO: not implemented yet. */
 	void loadColorsFromClicked();
 	/** Shows the "scale all symbols" dialog. */
@@ -321,9 +337,6 @@ public slots:
 	
 	/** Shows or hides the tags editor dock widget. */
 	void showTagsWindow(bool show);
-
-	/** Shows or hides the tag selection dock widget. */
-	void showTagSelectWindow(bool show);
 	
 	/** Shows the GeoreferencingDialog. */
 	void editGeoreferencing();
@@ -514,7 +527,7 @@ public slots:
 signals:
 	/**
 	 * @brief Indicates a change of the active symbol.
-	 * @param symbol The new active symbol, or NULL.
+	 * @param symbol The new active symbol, or nullptr.
 	 */
 	void activeSymbolChanged(const Symbol* symbol);
 	
@@ -569,15 +582,13 @@ private:
 	/// Updates enabled state of all widgets
 	void updateWidgets();
 	
-	void createSymbolWidget(QWidget* parent = NULL);
+	void createSymbolWidget(QWidget* parent = nullptr);
 	
 	void createColorWindow();
 	
 	void createTemplateWindow();
 	
 	void createTagEditor();
-
-	void createTagSelector();
 	
 	QAction* newAction(const char* id, const QString& tr_text, QObject* receiver, const char* slot, const char* icon = nullptr, const QString& tr_tip = QString{}, const char* whats_this_link = nullptr);
 	QAction* newCheckAction(const char* id, const QString& tr_text, QObject* receiver, const char* slot, const char* icon = nullptr, const QString& tr_tip = QString{}, const char* whats_this_link = nullptr);
@@ -589,7 +600,7 @@ private:
 	void createMobileGUI();
 	
 	void paintOnTemplate(Template* temp);
-	void updatePaintOnTemplateAction();
+	void finishPaintOnTemplate();
 	
 	void doUndo(bool redo);
 	
@@ -628,9 +639,11 @@ private:
 	QAction* select_nothing_act;
 	QAction* invert_selection_act;
 	QAction* select_by_current_symbol_act;
+	std::unique_ptr<MapFindFeature> find_feature;
 	QAction* clear_undo_redo_history_act;
 	
 	QAction* pan_act;
+	QAction* move_to_gps_pos_act;
 	QAction* zoom_in_act;
 	QAction* zoom_out_act;
 	QAction* show_all_act;
@@ -653,10 +666,12 @@ private:
 	QAction* scale_map_act;
 	QAction* rotate_map_act;
 	QAction* map_notes_act;
+	QAction* symbol_set_id_act;
 	
 	QAction* color_window_act;
 	QPointer<EditorDockWidget> color_dock_widget;
 	QAction* load_symbols_from_act;
+	QAction* load_crt_act;
 	
 	QAction* symbol_window_act;
 	EditorDockWidget* symbol_dock_widget;
@@ -670,9 +685,6 @@ private:
 	
 	QAction* tags_window_act;
 	QPointer<EditorDockWidget> tags_dock_widget;
-
-	QAction* tag_select_window_act;
-	QPointer<EditorDockWidget> tag_select_dock_widget;
 	
 	QAction* edit_tool_act;
 	QAction* edit_line_tool_act;
@@ -740,7 +752,6 @@ private:
 	
 	QFrame* statusbar_zoom_frame;
 	QLabel* statusbar_cursorpos_label;
-	QLabel* statusbar_objecttag_label;
 	QAction* copy_coords_act;
 	
 	QToolBar* toolbar_view;
@@ -754,6 +765,7 @@ private:
 	ActionGridBar* top_action_bar;
 	QToolButton* show_top_bar_button;
 	QAction* mobile_symbol_selector_action;
+	QMenu* mobile_symbol_button_menu;
 	
 	QComboBox* mappart_selector_box;
 	
@@ -776,4 +788,7 @@ Symbol* MapEditorController::activeSymbol() const
 	return active_symbol;
 }
 
+
+}  // namespace OpenOrienteering
+
 #endif
