@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2016  Kai Pastor
+ *    Copyright 2012-2017  Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -24,50 +24,84 @@
 #include "print_widget.h"
 
 #include <limits>
+#include <memory>
 
+#include <Qt>
+#include <QtGlobal>
+#include <QAbstractButton> // IWYU pragma: keep
 #include <QButtonGroup>
-#include <QDialogButtonBox>
+#include <QCheckBox>
 #include <QComboBox>
-#include <QFileDialog>
+#include <QDialog>
+#include <QDialogButtonBox>
+#include <QDoubleSpinBox>
+#include <QFile>
 #include <QFileInfo>
 #include <QFormLayout>
+#include <QHash>
 #include <QHBoxLayout>
+#include <QIcon>
+#include <QImage>
+#include <QLabel>
+#include <QLatin1Char>
+#include <QLatin1String>
+#include <QLayout>
 #include <QLineEdit>
+#include <QMargins>
 #include <QMessageBox>
+#include <QPagedPaintDevice>
 #include <QPainter>
+#include <QPointF>
+#include <QPrinterInfo>
 #include <QPrintDialog>
 #include <QPrintPreviewDialog>
 #include <QPushButton>
 #include <QRadioButton>
+#include <QRectF>
+#include <QRegExp>
+#include <QRegExpValidator>
 #include <QScrollArea>
 #include <QScrollBar>
+#include <QSignalBlocker>
+#include <QSizeF>
+#include <QSpacerItem>
+#include <QSpinBox>
+#include <QStringRef>
+#include <QStyle>
+#include <QStyleOption>
 #include <QToolButton>
+#include <QVBoxLayout>
+#include <QVariant>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
 #include <printer_properties.h>
 
-#include "main_window.h"
-#include "print_progress_dialog.h"
-#include "print_tool.h"
-#include "../core/map_printer.h"
+#include "core/georeferencing.h"
 #include "core/map.h"
+#include "core/map_printer.h"
+#include "core/map_view.h"
+#include "gui/file_dialog.h"
+#include "gui/main_window.h"
+#include "gui/print_progress_dialog.h"
+#include "gui/print_tool.h"
+#include "gui/util_gui.h"
 #include "gui/map/map_editor.h"
 #include "gui/map/map_widget.h"
-#include "../settings.h"
-#include "../templates/template.h"
-#include "util/util.h"
-#include "util_gui.h"
-#include "../util/backports.h"
-#include "../util/scoped_signals_blocker.h"
+#include "templates/template.h" // IWYU pragma: keep
+#include "templates/world_file.h"
+#include "util/backports.h"
+#include "util/scoped_signals_blocker.h"
 
 
-namespace
-{
+namespace OpenOrienteering {
+
+namespace {
+	
 	QToolButton* createPrintModeButton(const QIcon& icon, const QString& label, QWidget* parent = nullptr)
 	{
 		static const QSize icon_size(48,48);
-		QToolButton* button = new QToolButton(parent);
+		auto button = new QToolButton(parent);
 		button->setAutoRaise(true);
 		button->setCheckable(true);
 		button->setIconSize(icon_size);
@@ -76,7 +110,10 @@ namespace
 		button->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
 		return button;
 	}
-}
+	
+	
+}  // namespace
+
 
 //### PrintWidget ###
 
@@ -113,8 +150,8 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	paper_size_combo = new QComboBox();
 	layout->addRow(tr("Page format:"), paper_size_combo);
 	
-	QWidget* page_size_widget = new QWidget();
-	QHBoxLayout* page_size_layout = new QHBoxLayout();
+	auto page_size_widget = new QWidget();
+	auto page_size_layout = new QHBoxLayout();
 	page_size_widget->setLayout(page_size_layout);
 	page_size_layout->setMargin(0);
 	page_width_edit = Util::SpinBox::create(1, 0.1, 1000.0, tr("mm"), 1.0);
@@ -127,12 +164,12 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	layout->addRow({}, page_size_widget);
 	
 	page_orientation_widget = new QWidget();
-	QBoxLayout* page_orientation_layout = new QHBoxLayout();
+	auto page_orientation_layout = new QHBoxLayout();
 	page_orientation_layout->setContentsMargins(QMargins());
 	page_orientation_widget->setLayout(page_orientation_layout);
-	QRadioButton* portrait_button = new QRadioButton(tr("Portrait"));
+	auto portrait_button = new QRadioButton(tr("Portrait"));
 	page_orientation_layout->addWidget(portrait_button);
-	QRadioButton* landscape_button = new QRadioButton(tr("Landscape"));
+	auto landscape_button = new QRadioButton(tr("Landscape"));
 	page_orientation_layout->addWidget(landscape_button);
 	page_orientation_group = new QButtonGroup(this);
 	page_orientation_group->addButton(portrait_button, QPrinter::Portrait);
@@ -171,8 +208,8 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	
 	layout->addRow(Util::Headline::create(tr("Options")));
 	
-	QWidget* mode_widget = new QWidget();
-	QBoxLayout* mode_layout = new QHBoxLayout();
+	auto mode_widget = new QWidget();
+	auto mode_layout = new QHBoxLayout();
 	mode_widget->setLayout(mode_layout);
 	mode_layout->setMargin(0);
 	
@@ -181,7 +218,7 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	separations_mode_button = createPrintModeButton(QIcon(QString::fromLatin1(":/images/print-mode-separations.png")), tr("Color\nseparations"));
 	vector_mode_button->setChecked(true);
 	
-	QButtonGroup* mode_button_group = new QButtonGroup(this);
+	auto mode_button_group = new QButtonGroup(this);
 	mode_button_group->addButton(vector_mode_button);
 	mode_button_group->addButton(raster_mode_button);
 	mode_button_group->addButton(separations_mode_button);
@@ -214,7 +251,7 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	
 	// this must be created before its value is used to determine the default setting of page_orientation_combo
 	show_templates_check = new QCheckBox(tr("Show templates"));
-	QHBoxLayout* templates_warning_layout = new QHBoxLayout();
+	auto templates_warning_layout = new QHBoxLayout();
 	QIcon warning_icon = style()->standardIcon(QStyle::SP_MessageBoxWarning);
 	templates_warning_icon = new QLabel();
 	int pixmap_size = qBound(8, style()->pixelMetric(QStyle::PM_IndicatorHeight), 32);
@@ -229,6 +266,10 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	
 	overprinting_check = new QCheckBox(tr("Simulate overprinting"));
 	layout->addRow(overprinting_check);
+
+	world_file_check = new QCheckBox(tr("Save world file"));
+	layout->addRow(world_file_check);
+	world_file_check->hide();
 	
 	color_mode_combo = new QComboBox();
 	color_mode_combo->setEditable(false);
@@ -239,7 +280,7 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	scrolling_content = new QWidget();
 	scrolling_content->setLayout(layout);
 	
-	QBoxLayout* outer_layout = new QVBoxLayout();
+	auto outer_layout = new QVBoxLayout();
 	outer_layout->setContentsMargins(QMargins());
 	
 	scroll_area = new QScrollArea();
@@ -265,7 +306,7 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	export_button = new QPushButton(tr("Export..."));
 	export_button->hide();
 	button_box->addButton(export_button, QDialogButtonBox::ActionRole);
-	QPushButton* close_button = button_box->addButton(QDialogButtonBox::Close);
+	auto close_button = button_box->addButton(QDialogButtonBox::Close);
 	outer_layout->addWidget(button_box);
 	
 	setLayout(outer_layout);
@@ -286,6 +327,7 @@ PrintWidget::PrintWidget(Map* map, MainWindow* main_window, MapView* main_view,
 	
 	connect(mode_button_group, QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked), this, &PrintWidget::printModeChanged);
 	connect(dpi_combo->lineEdit(), &QLineEdit::textEdited, this, &PrintWidget::resolutionEdited);
+	connect(dpi_combo, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &PrintWidget::resolutionEdited);
 	connect(different_scale_check, &QAbstractButton::clicked, this, &PrintWidget::differentScaleClicked);
 	connect(different_scale_edit, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrintWidget::differentScaleEdited);
 	connect(show_templates_check, &QAbstractButton::clicked, this, &PrintWidget::showTemplatesClicked);
@@ -433,13 +475,13 @@ void PrintWidget::setActive(bool active)
 			if (zoom_to_map)
 			{
 				// Ensure the visibility of the whole map.
-				QRectF map_extent = map->calculateExtent(true, !main_view->areAllTemplatesHidden(), main_view);
+				auto map_extent = map->calculateExtent(true, !main_view->areAllTemplatesHidden(), main_view);
 				editor->getMainWidget()->ensureVisibilityOfRect(map_extent, MapWidget::ContinuousZoom);
 			}
 			else
 			{
 				// Ensure the visibility of the print area.
-				QRectF print_area(map_printer->getPrintArea());
+				auto print_area(map_printer->getPrintArea());
 				editor->getMainWidget()->ensureVisibilityOfRect(print_area, MapWidget::ContinuousZoom);
 			}
 			
@@ -474,7 +516,7 @@ void PrintWidget::setActive(bool active)
 void PrintWidget::updateTargets()
 {
 	QVariant current_target = target_combo->itemData(target_combo->currentIndex());
-	const QPrinterInfo* saved_printer = map_printer->getTarget();
+	const auto saved_printer = map_printer->getTarget();
 	const QString saved_printer_name = saved_printer ? saved_printer->printerName() : QString{};
 	int saved_target_index = -1;
 	int default_printer_index = -1;
@@ -490,17 +532,6 @@ void PrintWidget::updateTargets()
 			target_combo->setCurrentIndex(0);
 			
 			// Printers
-#if QT_VERSION < 0x050300
-			printers = QPrinterInfo::availablePrinters();
-			for (int i = 0; i < printers.size(); ++i)
-			{
-				if (printers[i].printerName() == saved_printer_name)
-					saved_target_index = target_combo->count();
-				if (printers[i].isDefault())
-					default_printer_index = target_combo->count();
-				target_combo->addItem(printers[i].printerName(), i);
-			}
-#else
 			auto default_printer_name = QPrinterInfo::defaultPrinterName();
 			printers = QPrinterInfo::availablePrinterNames();
 			for (int i = 0; i < printers.size(); ++i)
@@ -512,7 +543,6 @@ void PrintWidget::updateTargets()
 					default_printer_index = target_combo->count();
 				target_combo->addItem(name, i);
 			}
-#endif
 		}
 		
 		// Restore selected target if possible and exit on success
@@ -550,11 +580,7 @@ void PrintWidget::setTarget(const QPrinterInfo* target)
 	{
 		for (; target_index >= 0; target_index--)
 		{
-#if QT_VERSION < 0x050300
-			if (target && printers[target_index].printerName() == target->printerName())
-#else
 			if (target && printers[target_index] == target->printerName())
-#endif
 				break;
 			else if (!target)
 				break;
@@ -577,6 +603,11 @@ void PrintWidget::setTarget(const QPrinterInfo* target)
 	export_button->setDefault(!is_printer);
 	if (printer_properties_button)
 		printer_properties_button->setEnabled(is_printer);
+
+	world_file_check->setVisible(!is_printer);
+	// If MapCoord (0,0) maps to projected (0,0), then there is probably
+	// no point in writing a world file.
+	world_file_check->setChecked(!map->getGeoreferencing().toProjectedCoords(MapCoordF{}).isNull());
 	
 	bool is_image_target = target == MapPrinter::imageTarget();
 	vector_mode_button->setEnabled(!is_image_target);
@@ -605,14 +636,10 @@ void PrintWidget::targetChanged(int index) const
 	else if (target_index == ImageExporter)
 		map_printer->setTarget(MapPrinter::imageTarget());
 	else
-#if QT_VERSION < 0x050300
-		map_printer->setTarget(&printers[target_index]);
-#else
 	{
 		auto info = QPrinterInfo::printerInfo(printers[target_index]);
 		map_printer->setTarget(&info);
 	}
-#endif
 }
 
 // slot
@@ -725,7 +752,7 @@ void PrintWidget::applyPrintAreaPolicy() const
 	if (policy == SinglePage)
 	{
 		setOverlapEditEnabled(false);
-		QRectF print_area = map_printer->getPrintArea();
+		auto print_area = map_printer->getPrintArea();
 		print_area.setSize(map_printer->getPageRectPrintAreaSize());
 		if (center_check->isChecked())
 			centerOnMap(print_area);
@@ -742,7 +769,7 @@ void PrintWidget::applyCenterPolicy() const
 {
 	if (center_check->isChecked())
 	{
-		QRectF print_area = map_printer->getPrintArea();
+		auto print_area = map_printer->getPrintArea();
 		centerOnMap(print_area);
 		map_printer->setPrintArea(print_area);
 	}
@@ -750,9 +777,9 @@ void PrintWidget::applyCenterPolicy() const
 
 void PrintWidget::centerOnMap(QRectF& area) const
 {
-	QRectF map_extent = map->calculateExtent(false, show_templates_check->isChecked(), main_view);
-	area.moveLeft(map_extent.center().x() - 0.5f * area.width());
-	area.moveTop(map_extent.center().y() - 0.5f * area.height());
+	auto map_extent = map->calculateExtent(false, show_templates_check->isChecked(), main_view);
+	area.moveLeft(map_extent.center().x() - area.width() / 2);
+	area.moveTop(map_extent.center().y() - area.height() / 2);
 }
 
 
@@ -772,7 +799,7 @@ void PrintWidget::setPrintArea(const QRectF& area)
 	
 	if (center_check->isChecked())
 	{
-		QRectF centered_area = map_printer->getPrintArea();
+		auto centered_area = map_printer->getPrintArea();
 		centerOnMap(centered_area);
 		if ( qAbs(centered_area.left() - area.left()) > 0.005 ||
 			 qAbs(centered_area.top()  - area.top())  > 0.005 )
@@ -813,7 +840,7 @@ void PrintWidget::setPrintArea(const QRectF& area)
 // slot
 void PrintWidget::printAreaMoved()
 {
-	QRectF area = map_printer->getPrintArea();
+	auto area = map_printer->getPrintArea();
 	area.moveLeft(left_edit->value());
 	area.moveTop(-top_edit->value()); // Flip sign!
 	map_printer->setPrintArea(area);
@@ -822,7 +849,7 @@ void PrintWidget::printAreaMoved()
 // slot
 void PrintWidget::printAreaResized()
 {
-	QRectF area = map_printer->getPrintArea();
+	auto area = map_printer->getPrintArea();
 	area.setWidth(width_edit->value());
 	area.setHeight(height_edit->value());
 	map_printer->setPrintArea(area);
@@ -862,7 +889,7 @@ void PrintWidget::setOptions(const MapPrinterOptions& options)
 	switch (options.mode)
 	{
 	default:
-		Q_ASSERT(false && "Unhandled MapPrinterOptions::MapPrinterMode");
+		qWarning("Unhandled MapPrinterMode");
 		// fall through in release build
 	case MapPrinterOptions::Vector:
 		vector_mode_button->setChecked(true);
@@ -896,7 +923,7 @@ void PrintWidget::setOptions(const MapPrinterOptions& options)
 	switch (options.color_mode)
 	{
 	default:
-		Q_ASSERT(false && "Unhandled MapPrinterOptions::ColorMode");
+		qWarning("Unhandled ColorMode");
 		// fall through in release build
 	case MapPrinterOptions::DefaultColorMode:
 		color_mode_combo->setCurrentIndex(0);
@@ -952,6 +979,7 @@ void PrintWidget::updateResolutions(const QPrinterInfo* target) const
 	// Resolution list item with unit "dpi"
 	static QString dpi_template(QLatin1String("%1 ") + tr("dpi"));
 	QStringList resolutions;
+	resolutions.reserve(supported_resolutions.size());
 	for (auto resolution : qAsConst(supported_resolutions))
 		resolutions << dpi_template.arg(resolution);
 	
@@ -1138,7 +1166,7 @@ void PrintWidget::exportToImage()
 	                        filter_template.arg(tr("TIFF"), QString::fromLatin1("*.tif *.tiff")),
 	                        filter_template.arg(tr("JPEG"), QString::fromLatin1("*.jpg *.jpeg")),
 	                        tr("All files (*.*)") };
-	QString path = QFileDialog::getSaveFileName(this, tr("Export map ..."), {}, filters.join(QString::fromLatin1(";;")));
+	QString path = FileDialog::getSaveFileName(this, tr("Export map ..."), {}, filters.join(QString::fromLatin1(";;")));
 	if (path.isEmpty())
 		return;
 	
@@ -1180,11 +1208,33 @@ void PrintWidget::exportToImage()
 	else
 	{
 		main_window->showStatusBarMessage(tr("Exported successfully to %1").arg(path), 4000);
+		if (world_file_check->isChecked())
+			exportWorldFile(path);  /// \todo Handle errors
 		emit finished(0);
 	}
 	return;
 }
 
+void PrintWidget::exportWorldFile(const QString& path) const
+{
+	const auto& georef = map->getGeoreferencing();
+	const auto& ref_transform = georef.mapToProjected();
+
+	qreal pixel_per_mm = (map_printer->getOptions().resolution / 25.4) * map_printer->getScaleAdjustment();
+	const auto center_of_pixel = QPointF{0.5/pixel_per_mm, 0.5/pixel_per_mm};
+	const auto top_left = georef.toProjectedCoords(MapCoord{map_printer->getPrintArea().topLeft() + center_of_pixel});
+
+
+	const auto xscale = ref_transform.m11() / pixel_per_mm;
+	const auto yscale = ref_transform.m22() / pixel_per_mm;
+	const auto xskew  = ref_transform.m12() / pixel_per_mm;
+	const auto yskew  = ref_transform.m21() / pixel_per_mm;
+
+	QTransform final_wld(xscale, yskew, 0, xskew, yscale, 0, top_left.x(), top_left.y());
+	WorldFile wld(final_wld);
+	wld.save(WorldFile::pathForImage(path));
+}
+
 void PrintWidget::exportToPdf()
 {
 	auto printer = map_printer->makePrinter();
@@ -1202,7 +1252,7 @@ void PrintWidget::exportToPdf()
 	static const QString filter_template(QString::fromLatin1("%1 (%2)"));
 	QStringList filters = { filter_template.arg(tr("PDF"), QString::fromLatin1("*.pdf")),
 	                        tr("All files (*.*)") };
-	QString path = QFileDialog::getSaveFileName(this, tr("Export map ..."), {}, filters.join(QString::fromLatin1(";;")));
+	QString path = FileDialog::getSaveFileName(this, tr("Export map ..."), {}, filters.join(QString::fromLatin1(";;")));
 	if (path.isEmpty())
 	{
 		return;
@@ -1336,4 +1386,7 @@ bool PrintWidget::checkForEmptyMap()
 	return false;
 }
 
-#endif
+
+}  // namespace OpenOrienteering
+
+#endif  // QT_PRINTSUPPORT_LIB
