@@ -21,14 +21,22 @@
 #ifndef OPENORIENTEERING_GENERAL_SETTINGS_PAGE_H
 #define OPENORIENTEERING_GENERAL_SETTINGS_PAGE_H
 
+#include <QObject>
+#include <QString>
+#include <QVariant>
+
 #include "settings_page.h"
 
 class QCheckBox;
 class QComboBox;
 class QDoubleSpinBox;
+class QEvent;
 class QSpinBox;
+class QWidget;
 
 
+namespace OpenOrienteering {
+
 class GeneralSettingsPage : public SettingsPage
 {
 Q_OBJECT
@@ -76,6 +84,7 @@ private:
 	QCheckBox* tips_visible_check;
 	
 	QCheckBox* compatibility_check;
+	QCheckBox* undo_check;
 	QCheckBox* autosave_check;
 	QSpinBox*  autosave_interval_edit;
 	
@@ -83,4 +92,7 @@ private:
 	QCheckBox* ocd_importer_check;
 };
 
+
+}  // namespace OpenOrienteering
+
 #endif
