@@ -24,8 +24,16 @@
 #include <QtGlobal>
 
 #if QT_VERSION < 0x050700
-#  include "qasconst.h"
-#  include "qoverload.h"
+#  include "qasconst.h"  // IWYU pragma: export
+#  include "qoverload.h" // IWYU pragma: export
+#endif
+
+#if QT_VERSION < 0x050500
+#  if defined(QT_NO_INFO_OUTPUT)
+#    define qInfo QT_NO_QDEBUG_MACRO
+#  else
+#    define qInfo qDebug
+#  endif
 #endif
 
 #if defined(Q_OS_ANDROID) && defined(_GLIBCXX_CMATH)
