@@ -20,6 +20,12 @@
 
 #include "scoped_signals_blocker.h"
 
+#include <QtGlobal>
+#include <QObject>
+
+
+namespace OpenOrienteering {
+
 ScopedMultiSignalsBlocker::~ScopedMultiSignalsBlocker()
 {
 #ifdef MAPPER_DEVELOPMENT_BUILD
@@ -41,3 +47,6 @@ void ScopedMultiSignalsBlocker::add(QObject* object)
 	items.push_back( {object, object && object->blockSignals(true)} );
 	Q_ASSERT(!object || object->signalsBlocked());
 }
+
+
+}  // namespace OpenOrienteering
