@@ -19,10 +19,14 @@
 
 #include "georeferencing_t.h"
 
+#include <QtTest>
+
 #include <proj_api.h>
 
-#include "../src/core/crs_template.h"
-#include "../src/fileformats/xml_file_format.h"
+#include "core/crs_template.h"
+#include "fileformats/xml_file_format.h"
+
+using namespace OpenOrienteering;
 
 
 int XMLFileFormat::active_version = 6;
@@ -30,6 +34,7 @@ int XMLFileFormat::active_version = 6;
 
 namespace
 {
+	// clazy:excludeall=non-pod-global-static
 	QString epsg5514_spec = QLatin1String("+init=epsg:5514");
 	QString gk2_spec   = QLatin1String("+proj=tmerc +lat_0=0 +lon_0=6 +k=1.000000 +x_0=2500000 +y_0=0 +ellps=bessel +datum=potsdam +units=m +no_defs");
 	QString gk3_spec   = QLatin1String("+proj=tmerc +lat_0=0 +lon_0=9 +k=1.000000 +x_0=3500000 +y_0=0 +ellps=bessel +datum=potsdam +units=m +no_defs");
