@@ -20,10 +20,17 @@
 
 #include "locale_t.h"
 
-#include <QtTest/QtTest>
+#include <QtTest>
+#include <QLatin1String>
+#include <QLocale>
+#include <QString>
 
 #include "util/translation_util.h"
 
+using namespace OpenOrienteering;
+
+
+namespace OpenOrienteering {
 
 inline
 bool operator==(const TranslationUtil::Language& first, const TranslationUtil::Language& second) {
@@ -31,6 +38,10 @@ bool operator==(const TranslationUtil::Language& first, const TranslationUtil::L
 }
 
 
+}  // namespace OpenOrienteering
+
+
+
 LocaleTest::LocaleTest(QObject* parent)
 : QObject(parent)
 {
